# -*- coding:utf-8 -*-
# Created by Hans-Thomas on 2012-09-02.
#=============================================================================
#   autocheck.py --- Autocheck command
#=============================================================================
from __future__ import absolute_import, print_function

import os, sys
from optparse import make_option

import django
from django.core.management import BaseCommand


class Command(BaseCommand):
    help = 'Run tests automatically'
    option_list = (
        make_option('--once', action='store_true', default=False,
            help='Single test run.'
        ),
        make_option('--no-verbose', action='store_false', dest='verbose', default=True,
            help='Do not run unittests verbose.'
        ),
        make_option('--noinput', action='store_false', dest='interactive', default=True,
            help='Tells Django to NOT prompt the user for input of any kind.'
        ),
        make_option('-f', '--failfast', action='store_true', default=False,
            help='Stop on first failure.'
        ),
        make_option('-c', '--catch', action='store_true', default=False,
            help='Catch control-C and display results.'
        ),
        make_option('--no-buffer', action='store_false', dest='buffer', default=True,
            help='Do not buffer stdout and stderr during test runs.'
        ),
        make_option('-s', metavar='DIRECTORY', dest='start', default='.',
            help='Directory to start discovery (default: ".").'
        ),
        make_option('-p', metavar='PATTERN', dest='pattern',
            help='Pattern to match test files ("test*.py" default).'
        ),
        make_option('-t', metavar='DIRECTORY', dest='top_level', default='.',
            help='Top level directory of project (default: ".").'
        ),
    ) + BaseCommand.option_list
    if django.get_version() < '1.7':
        requires_model_validation = False
    else:
        requires_system_checks = False
    can_import_settings = False
    leave_locale_alone = True
    
    def handle(self, *args, **options):
        if options.pop('once'):
            self.once(*args, **options)
        else:
            self.autotest(*args, **options)
    
    def once(self, *args, **options):
        from autocheck.main import Unbuffered
        sys.stdout = Unbuffered(sys.stdout)
        from django.conf import settings
        from django.test.utils import get_runner
        verbosity = int(options.pop('verbosity'))
        TestRunner = get_runner(settings)
        test_runner = TestRunner(verbosity=verbosity, **options)
        failures = test_runner.run_tests(args, **options)
        if failures:
            sys.exit(bool(failures))
    
    def autotest(self, *args, **options):
        import resource
        try:
            resource.setrlimit(resource.RLIMIT_NOFILE, (4096, -1))
        except Exception, e:
            print(repr(e))
        settings = options['settings']
        if not settings and os.path.exists('test_settings.py'):
            settings = 'test_settings'
        os.environ['DJANGO_SETTINGS_MODULE'] = settings
        from autocheck.main import autocheck
        autocheck(sys.argv)


#.............................................................................
#   autocheck.py



# -*- coding:utf-8 -*-
# Created by Hans-Thomas on 2011-06-01.
#=============================================================================
#   tags.py --- Add tags to tests
#=============================================================================
from .compat import unittest


def tag(*tags):
    def decorator(test_item):
        test_item.__autocheck_tags__ = set(tags)
        return test_item
    return decorator

def add_tags(test_item, tags):
    all_tags = tuple(get_tags(test_item) | set(tags))
    return tag(*all_tags)(test_item)

def get_tags(test_item):
    if isinstance(test_item, unittest.TestCase):
        try:
            return test_item.__autocheck_tags__
        except AttributeError:
            test_item = getattr(test_item, test_item._testMethodName)
    return getattr(test_item, '__autocheck_tags__', set())


#.............................................................................
#   tags.py



# -*- coding:utf-8 -*-
# Created by Hans-Thomas on 2011-05-20.
#=============================================================================
#   testsuiterunner.py --- Django test suite runner
#=============================================================================
from django.test.simple import DjangoTestSuiteRunner

from .quickstart import QuickstartMixin
from autocheck.db import Database
from autocheck.testrunner import TestRunner, TestProgram


class TestSuiteRunner(QuickstartMixin, DjangoTestSuiteRunner):
    opts = dict(
        failfast = '--failfast',
        catch = '--catch',
        buffer = '--buffer',
        pattern = '-p',
        verbose = '-v',
    )
    discover_opts = dict(
        start = '-s',
        top_level = '-t',
    )
    discover_opts.update(opts)

    def run_tests(self, test_labels, extra_tests=None, **kwargs):
        argv = ['./manage.py unittest']
        discover = len(test_labels) == 0 or len(test_labels) == 1 and test_labels[0] == 'discover'
        if discover:
            argv.append('discover')
            opts = self.discover_opts
        else:
            opts = self.opts
        for key, value in kwargs.items():
            if key in opts:
                option = opts[key]
                if value is True:
                    argv.append(option)
                elif value is not False and value is not None:
                    argv.append(option)
                    argv.append(value)
        if not discover:
            argv.extend(test_labels)
        self.setup_test_environment()
        old_config = self.setup_databases()
        result = TestProgram(module=None, argv=argv, testRunner=TestRunner, database=Database())
        self.teardown_databases(old_config)
        self.teardown_test_environment()
        return self.suite_result(suite, result)

#.............................................................................
#   testsuiterunner.py
